package net.sergeych.crypto2

import kotlinx.serialization.Serializable

/**
 * Key Identity. Can be used to find matching keys for decryption or verifying, etc. The identity
 * may contain [KDF] parameters if the corresponding key could be derived from a password.
 * Note that [kdf] part is not respected in [equals].
 *
 * Important. `KeyId` of matching keys are the same, so you can use it to identify
 * and find matching keys in the [UniversalRing], etc. For example:
 *
 * - [Asymmetric.SecretKey] and [Asymmetric.PublicKey] from the same pair have the same `KeyId`, thus the former
 *   can decrypt what was encrypted with the latter.
 *
 * - [SigningSecretKey] and corresponding [VerifyingKey] have the same `KeyId`. Use it to pick a proper key for
 *   signing from a ring with [UniversalRing.findKey]
 *
 */
@Serializable
data class KeyId(val id: BinaryId, val kdf: KeyDerivationParams?=null ) {

    /**
     * Binary array representation of the [id], not including the [kdf]. Used in [SafeKeyExchange]
     * and other key exchanges to generate session tokens, etc.
     */
    val binaryTag: UByteArray by lazy { id.id }

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other !is KeyId) return false
        return id == other.id
    }

    override fun hashCode(): Int {
        return id.hashCode()
    }

    override fun toString() = id.toString()

    constructor(magickNumber: KeysMagickNumber, keyData: UByteArray, kdp: KeyDerivationParams?=null)
    : this(BinaryId.createFromUBytes(magickNumber.number, blake2b3l(keyData)), kdp)
}